@@ -4,6 +4,14 @@ from pycuda.compiler import SourceModule
 import numpy as np
 import math
 import time
+import lowpy as lp
+import os.path
+# For importing the .cu file from the lowpy package:
+try:
+    import importlib.resources as pkg_resources
+except ImportError:
+    # Try backported to PY<37 `importlib_resources`.
+    import importlib_resources as pkg_resources
 
 # Start timer from 0
 def tic():
@@ -72,83 +80,14 @@ class Dense:
         self.hits_h     = np.zeros(self.J_h,dtype=np.float64)
         self.hits_d     = self.hostToDevice(self.hits_h)
         # Cuda programs
-        self.program = SourceModule("""
-            __global__ void propagate(
-                const   int                     I,
-                const   double * __restrict__   x,
-                const   double * __restrict__   w,
-                const   double * __restrict__   b,
-                        double * __restrict__   y,
-                        double * __restrict__   z
-            ){
-                int j = blockIdx.x;
-                double sum = 0;
-                for (int i = 0; i < I; i++) sum += x[i]*w[j*I+i];
-                y[j] = sum + b[j];
-                z[j] = 1/(1 + exp(-y[j]));
-            }
-            __global__ void backpropagate(
-                const   int                     label,
-                        double * __restrict__   dedz,
-                const   double * __restrict__   z,
-                const   int                     J_n,
-                        double * __restrict__   w_n,
-                const   double * __restrict__   dedz_n,
-                const   double * __restrict__   dzdy_n,
-                        double * __restrict__   dzdy,
-                const   double                  alpha,
-                const   int                     I,
-                        double * __restrict__   b,
-                        double * __restrict__   w,
-                const   double * __restrict__   x,
-                const   double                  beta,
-                        double * __restrict__   vtb,
-                        double * __restrict__   vtw
-            ){
-                int j   = blockIdx.x;
-                int I_n = gridDim.x;
-                if (label > -1){
-                    if (j == label){
-                        dedz[j] = z[j] - 1;
-                    }else{
-                        dedz[j] = z[j] - 0;
-                    }
-                }else{
-                    double sum = 0;
-                    for (int j_n = 0; j_n < J_n; j_n++) sum += w_n[j+j_n*I_n] * dedz_n[j_n] * dzdy_n[j_n];
-                    dedz[j] = sum;
-                }
-                dzdy[j] = z[j] * (1 - z[j]);
-                b[j]   -= (beta * vtb[j] + alpha * dedz[j] * dzdy[j]);
-                vtb[j]  = beta * vtb[j] + alpha * dedz[j] * dzdy[j];
-                for (int i = 0; i < I; i++){
-                    w[j*I+i]   -= (beta * vtw[j*I+i] + alpha * dedz[j] * dzdy[j] * x[i]);
-                    vtw[j*I+i]  = beta * vtw[j*I+i] + alpha * dedz[j] * dzdy[j] * x[i];
-                }
-            }
-            __global__ void argmax(
-                    const   int                     label,
-                    const   double * __restrict__   z,
-                            double *__restrict__    hits
-            ){
-                int j = blockIdx.x;
-                if (j == 0){
-                    double maxVal = 0;
-                    int maxIdx = 0;
-                    for (int i = 0; i < 10; i++){
-                        if (z[i] > maxVal){
-                            maxIdx = i;
-                            maxVal = z[i];
-                        }
-                    }
-                    if (maxIdx == label){
-                        hits[0] += 1;
-                    }
-                    hits[1] = maxIdx;
-                    hits[2] = maxVal;
-                }
-            }
-        """)
+        # try:
+        #     self.program = SourceModule(open('lowpy/dense.cu','r').read())
+        # except FileNotFoundError:
+        #     self.program = SourceModule(pkg_resources.read_text(lp, 'dense.cu'))
+        if os.path.isfile('lowpy/dense.cu'):# Differentiates between package cuda file and local
+            self.program = SourceModule(open('lowpy/dense.cu','r').read())
+        else:
+            self.program = SourceModule(pkg_resources.read_text(lp, 'dense.cu'))
         self.propagateFunction = self.program.get_function("propagate")
         self.propagateFunction.prepare("PPPPPP")
         self.backpropagateFunction = self.program.get_function("backpropagate")
